   program define sem4gmm
      syntax varlist if, at(name) obsvar(varlist)
      // varlist is the list of vectorized residuals
      // at is the name of the parameter vector

      // create a variable to address the `if' condition
      marksample touse, novarlist

      // generate the implied matrix
      tempname Sigma
      mata : ParsToSigma( "`at'", "`Sigma'" )

      // produce vectorized deviations
      unab thevars : `obsvar'
      mata : VechData("`thevars'", "`varlist'", 1, "`touse'", st_matrix("`Sigma'") )
   end

   mata:

   void VechData( string scalar thevars, string scalar vechlist, | ///
      real scalar dev, string scalar tousename, real matrix Sigma) {

      // number of observed variables
      p = length( tokens( thevars ) )

      // center the variables before vech?
      if (missing(dev)) dev = 0;

      // subtract the supplied matrix?
      if (rows(Sigma)==0 | rows(Sigma)!=p ) Sigma = J( p, p, 0);

      // retrieve the data
      st_view(x=.,.,tokens(thevars), tousename )
      m = mean( x,1):/rows(x):*dev

      // retrieve the variables for vectorization
      st_view(v=.,.,tokens(vechlist), tousename )

      // compute vectorization observation by observation
      for( i=1;i<=rows(x); i++) {
         v[i,] = vech( (x[i,]-m)'*(x[i,]-m ) )' - vech(Sigma)'
      }

   }

   void ParsToSigma(string scalar atName, string scalar SigmaName) {

      real vector at, VarZeta;
      real matrix Lambda, Phi, Gamma, Psi, CovXX, CovXY, CovYY;

      // obtain the vector of parameters from Stata vector `at'
      at = st_matrix( atName )

      // check the number of parameters
      if (cols(at)!=24) {
         st_matrix( SigmaName, J(0,0,.) )
         return
      }

      // set up the parameter matrices
      // loadings:
      Lambda = (1, at[1..4])'

      // covariances of the exogenous variables
      Phi = invvech( at[5..14]' )

      // variance of the latent regression term
      VarZeta = at[15]

      // regression coefficients
      Gamma = at[16..19]

      // measurement errors
      Psi  = diag( at[20..24] )

      // covariance of the exogenous variables goes as is
      CovXX = Phi

      // covariance between the explanatory variables and indicators
      CovXY = Phi * Gamma' * Lambda'

      // covariance of the indicators
      CovYY = Lambda * Gamma * Phi * Gamma' * Lambda' + ///
              Lambda * VarZeta * Lambda' + Psi

      // return the matrix back to Stata
      st_matrix(SigmaName, (CovXX, CovXY \ CovXY', CovYY) )
   }

   printf("{inp}.")
   end
   // of mata piece
